﻿using System;
using System.Runtime.InteropServices;

namespace OpenLibSys
{
    public class Ols : IDisposable
    {
        const string dllName64 = "cpuid64.dll";
        const string dllName32 = "cpuid32.dll";

        // for this support library
        public enum Status
        {
            NO_ERROR = 0,
            DLL_NOT_FOUND = 1,
            DLL_INCORRECT_VERSION = 2,
            DLL_INITIALIZE_ERROR = 3,
        }

        [DllImport("kernel32")]
        public extern static IntPtr LoadLibrary(string lpFileName);

        [DllImport("kernel32", SetLastError = true)]
        private static extern bool FreeLibrary(IntPtr hModule);

        [DllImport("kernel32", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = false)]
        private static extern IntPtr GetProcAddress(IntPtr hModule, [MarshalAs(UnmanagedType.LPStr)]  string lpProcName);

        private IntPtr module = IntPtr.Zero;
        private uint status = (uint)Status.NO_ERROR;

        public Ols()
        {
            string fileName;

            if (System.IntPtr.Size == 8)
            {
                fileName = dllName64;
            }
            else
            {
                fileName = dllName32;
            }

            module = Ols.LoadLibrary(fileName);
            if (module == IntPtr.Zero)
            {
                status = (uint)Status.DLL_NOT_FOUND;
            }
            else
            { 
                Cpuid = (_Cpuid)GetDelegate("Cpuid", typeof(_Cpuid));

                if (Cpuid == null )
                {
                    status = (uint)Status.DLL_INCORRECT_VERSION;
                }
            }
        }

        public uint GetStatus()
        {
            return status;
        }

        public void Dispose()
        {
            if (module != IntPtr.Zero)
            {
                Ols.FreeLibrary(module);
                module = IntPtr.Zero;
            }
        }

        public Delegate GetDelegate(string procName, Type delegateType)
        {
            IntPtr ptr = GetProcAddress(module, procName);
            if (ptr != IntPtr.Zero)
            {
                Delegate d = Marshal.GetDelegateForFunctionPointer(ptr, delegateType);
                return d;
            }

            int result = Marshal.GetHRForLastWin32Error();
            throw Marshal.GetExceptionForHR(result);
        }

        //-----------------------------------------------------------------------------
        // CPU
        //-----------------------------------------------------------------------------
        public delegate int _Cpuid(uint indexEax, uint indexEcx, ref uint eax, ref uint ebx, ref uint ecx, ref uint edx);
       
        public _Cpuid Cpuid = null;
    }
}
